#version 460 core

layout(location = 0) out vec4 texBuffer3;
layout(location = 1) out vec4 texBuffer4;

uniform vec2 screenSize;
uniform vec2 screenTexel;
uniform sampler2D colortex3;

in vec2 Texcoord;

float GetLuminance(in vec3 x)
{
    return dot(x, vec3(0.2126, 0.7152, 0.0722));
}

#define FXAA

#ifdef FXAA

float quality[12] = float[12] (1.0, 1.0, 1.0, 1.0, 1.0, 1.5, 2.0, 2.0, 2.0, 2.0, 4.0, 8.0);

void FXAA311(inout vec3 color)
{
	const float edgeThresholdMin = 0.03125;
	const float edgeThresholdMax = 0.125;
	const float subpixelQuality = 0.75;
	const int iterations = 12;
	
	vec2 view = screenTexel;
	
	float lumaCenter = GetLuminance(color);
	float lumaDown  = GetLuminance(texture(colortex3, Texcoord + vec2( 0.0, -view.y)).rgb);
	float lumaUp    = GetLuminance(texture(colortex3, Texcoord + vec2( 0.0, view.y)).rgb);
	float lumaLeft  = GetLuminance(texture(colortex3, Texcoord + vec2(-view.x,  0.0)).rgb);
	float lumaRight = GetLuminance(texture(colortex3, Texcoord + vec2(view.x, 0.0)).rgb);
	
	float lumaMin = min(lumaCenter, min(min(lumaDown, lumaUp), min(lumaLeft, lumaRight)));
	float lumaMax = max(lumaCenter, max(max(lumaDown, lumaUp), max(lumaLeft, lumaRight)));
	
	float lumaRange = lumaMax - lumaMin;
	
	if (lumaRange > max(edgeThresholdMin, lumaMax * edgeThresholdMax)) {
		float lumaDownLeft  = GetLuminance(texture(colortex3, Texcoord - view).rgb);
		float lumaUpRight   = GetLuminance(texture(colortex3, Texcoord + view).rgb);
		float lumaUpLeft    = GetLuminance(texture(colortex3, Texcoord + vec2(-view.x, view.y)).rgb);
		float lumaDownRight = GetLuminance(texture(colortex3, Texcoord + vec2(view.x, -view.y)).rgb);
		
		float lumaDownUp    = lumaDown + lumaUp;
		float lumaLeftRight = lumaLeft + lumaRight;
		
		float lumaLeftCorners  = lumaDownLeft  + lumaUpLeft;
		float lumaDownCorners  = lumaDownLeft  + lumaDownRight;
		float lumaRightCorners = lumaDownRight + lumaUpRight;
		float lumaUpCorners    = lumaUpRight   + lumaUpLeft;
		
		float edgeHorizontal = abs(-2.0 * lumaLeft   + lumaLeftCorners ) +
							   abs(-2.0 * lumaCenter + lumaDownUp      ) * 2.0 +
							   abs(-2.0 * lumaRight  + lumaRightCorners);
		float edgeVertical   = abs(-2.0 * lumaUp     + lumaUpCorners   ) +
							   abs(-2.0 * lumaCenter + lumaLeftRight   ) * 2.0 +
							   abs(-2.0 * lumaDown   + lumaDownCorners );
		
		bool isHorizontal = (edgeHorizontal >= edgeVertical);		
		
		float luma1 = isHorizontal ? lumaDown : lumaLeft;
		float luma2 = isHorizontal ? lumaUp : lumaRight;
		float gradient1 = luma1 - lumaCenter;
		float gradient2 = luma2 - lumaCenter;
		
		bool is1Steepest = abs(gradient1) >= abs(gradient2);
		float gradientScaled = 0.25 * max(abs(gradient1), abs(gradient2));
		
		float stepLength = isHorizontal ? view.y : view.x;

		float lumaLocalAverage = 0.0;

		if (is1Steepest) {
			stepLength = - stepLength;
			lumaLocalAverage = 0.5 * (luma1 + lumaCenter);
		} else {
			lumaLocalAverage = 0.5 * (luma2 + lumaCenter);
		}
		
		vec2 currentUv = Texcoord;
		if(isHorizontal)
			currentUv.y += stepLength * 0.5;
		else
			currentUv.x += stepLength * 0.5;
		
		vec2 offset = isHorizontal ? vec2(view.x, 0.0) : vec2(0.0, view.y);
		
		vec2 uv1 = currentUv - offset;
		vec2 uv2 = currentUv + offset;

		float lumaEnd1 = GetLuminance(texture(colortex3, uv1).rgb);
		float lumaEnd2 = GetLuminance(texture(colortex3, uv2).rgb);
		lumaEnd1 -= lumaLocalAverage;
		lumaEnd2 -= lumaLocalAverage;
		
		bool reached1 = abs(lumaEnd1) >= gradientScaled;
		bool reached2 = abs(lumaEnd2) >= gradientScaled;
		bool reachedBoth = reached1 && reached2;
		
		if (!reached1)
			uv1 -= offset;
		if (!reached2)
			uv2 += offset;
		
		if (!reachedBoth) {
			for(int i = 2; i < iterations; i++) {
				if (!reached1) {
					lumaEnd1 = GetLuminance(texture(colortex3, uv1).rgb);
					lumaEnd1 = lumaEnd1 - lumaLocalAverage;
				}
				if (!reached2) {
					lumaEnd2 = GetLuminance(texture(colortex3, uv2).rgb);
					lumaEnd2 = lumaEnd2 - lumaLocalAverage;
				}
				
				reached1 = abs(lumaEnd1) >= gradientScaled;
				reached2 = abs(lumaEnd2) >= gradientScaled;
				reachedBoth = reached1 && reached2;

				if (!reached1)
					uv1 -= offset * quality[i];
				if (!reached2)
					uv2 += offset * quality[i];
				if (reachedBoth) break;
			}
		}
		
		float distance1 = isHorizontal ? (Texcoord.x - uv1.x) : (Texcoord.y - uv1.y);
		float distance2 = isHorizontal ? (uv2.x - Texcoord.x) : (uv2.y - Texcoord.y);

		bool isDirection1 = distance1 < distance2;
		float distanceFinal = min(distance1, distance2);

		float edgeThickness = (distance1 + distance2);

		float pixelOffset = - distanceFinal / edgeThickness + 0.5f;
		
		bool isLumaCenterSmaller = lumaCenter < lumaLocalAverage;

		bool correctVariation = ((isDirection1 ? lumaEnd1 : lumaEnd2) < 0.0) != isLumaCenterSmaller;

		float finalOffset = correctVariation ? pixelOffset : 0.0;
		
		float lumaAverage = (1.0 / 12.0) * (2.0 * (lumaDownUp + lumaLeftRight) + lumaLeftCorners + lumaRightCorners);
		float subPixelOffset1 = clamp(abs(lumaAverage - lumaCenter) / lumaRange, 0.0, 1.0);
		float subPixelOffset2 = (-2.0 * subPixelOffset1 + 3.0) * subPixelOffset1 * subPixelOffset1;
		float subPixelOffsetFinal = subPixelOffset2 * subPixelOffset2 * subpixelQuality;

		finalOffset = max(finalOffset, subPixelOffsetFinal);
		
		
		// Compute the final UV coordinates.
		vec2 finalUv = Texcoord;
		if (isHorizontal)
			finalUv.y += finalOffset * stepLength;
		else
			finalUv.x += finalOffset * stepLength;

		color = texture(colortex3, finalUv).rgb;
	}
}
#endif

vec3 colorFetch(in vec2 uv, const int b)
{
	if(uv.x > 1.0 || uv.x < 0.0 || uv.y > 1.0 || uv.x < 0.0)
		return vec3(0.0);
	vec3 color;
	for(int i = 0;i < b;i++)
	{
		for(int j = 0;j < b;j++)
		{
			color += texture(colortex3, uv + (vec2(i, j) / screenSize)).xyz;
		}
	}
	return color / (b * b);
}

//#define BETTER_BLOOM

void main()
{
    vec3 AA_Color = texture(colortex3, Texcoord).rgb;
	#ifdef FXAA
		FXAA311(AA_Color);
	#endif
    texBuffer3 = vec4(AA_Color, 1.0);
    
	vec3 bloom;
	#ifdef BETTER_BLOOM
	bloom += colorFetch(Texcoord * 2.0, 2);
	bloom += colorFetch((Texcoord - vec2(1.0 / 2.0 + screenTexel.x * 10.0, 0.0)) * 4.0, 4);
	bloom += colorFetch((Texcoord - vec2(1.0 / 2.0 + 1.0 / 4.0 + screenTexel.x * 20.0, 0.0)) * 8.0, 8);
	bloom += colorFetch((Texcoord - vec2(1.0 / 2.0 + 1.0 / 4.0 + 1.0 / 8.0 + screenTexel.x * 30.0, 0.0)) * 16.0, 16);
	#else
	bloom = texture(colortex3, Texcoord).rgb;
	bloom = bloom * bloom;
	#endif
	bloom = bloom * bloom;
	texBuffer4 = vec4(bloom, 1.0);
}
/* DRAWBUFFERS:34 */