#version 460 core

layout(location = 0) out vec4 texBuffer3;
layout(location = 1) out vec4 texBuffer5;

uniform int frameCounter;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform float far;
uniform float near;
uniform vec2 screenSize;
uniform vec2 screenTexel;
uniform sampler2D gaux4;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D colortex5;

const bool colortex5Clear = false;

in vec2 Texcoord;

#include "/HmwsLib/BasicFunctions.glsl"
#include "/HmwsLib/SpaceTransform.glsl"

#define TAA

#ifdef TAA

const vec2 prevCoordOffset[8] =
{
	vec2(1.0, 1.0),
	vec2(1.0, 0.0),
	vec2(1.0, -1.0),
	vec2(0.0, 1.0),
	vec2(0.0, -1.0),
	vec2(-1.0, 1.0),
	vec2(-1.0, 0.0),
	vec2(-1.0, -1.0)
};

vec2 ToPrevious(in vec2 pos0)
{
	vec4 viewPosPrev = iProjDiag * vec4(pos0 * 2.0 - 1.0, 0.0, 1.0) + gbufferProjectionInverse[3];
	viewPosPrev.xyz = viewMat3Inverse * viewPosPrev.xyz + gbufferModelViewInverse[3].xyz;
	
	vec4 previousPosition = viewPosPrev + vec4(cameraPosition - previousCameraPosition, 0.0);
	previousPosition.xyz = viewMat3Prev * previousPosition.xyz + gbufferPreviousModelView[3].xyz;
    previousPosition.xyz = (prevprojd * previousPosition.xyz + gbufferPreviousProjection[3].xyz) / -previousPosition.z;
    return previousPosition.xy * 0.5 + 0.5;
}

vec2 prvCoord = ToPrevious(Texcoord);

float luma(vec3 color)
{
	return dot(color, vec3(0.21, 0.72, 0.07));
}

vec3 tonemap(vec3 col)
{
	return col / (1.0 + luma(col));
}
vec3 invTonemap(vec3 col)
{
	return col / (1.0 - luma(col));
}

void DoTAA(inout vec3 color)
{
	vec3 tempColor = texture(colortex5, Texcoord).rgb;
	
	vec3 maxc = color;
	vec3 minc = color;

	vec3 samp = texture(colortex3, Texcoord + screenTexel).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(0.0, screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(-screenTexel.x, screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(screenTexel.x, 0.0)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(-screenTexel.x, 0.0)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(screenTexel.x, -screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord + vec2(0.0, -screenTexel.y)).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);
	samp = texture(colortex3, Texcoord - screenTexel).rgb;
	maxc = max(maxc, samp);
	minc = min(minc, samp);

	tempColor = clamp(tempColor, minc, maxc);

	color = invTonemap(mix(tonemap(color), tonemap(tempColor), exp(-length((Texcoord - prvCoord.xy) * screenSize)) * 0.6 + 0.3125));
}
#endif

//#define MOTION_BLUR
#define MOTION_BLUR_QUALITY 8

#ifdef MOTION_BLUR

void motionBlur(inout vec3 color)
{
	vec2 move = (prvCoord - Texcoord);
	float len = min(length(move) / MOTION_BLUR_QUALITY * 0.2, 0.1);
	vec2 dir = normalize(move) * len;
	vec2 point = Texcoord;
	if(len > 0.0)
	{
		for(int i = 0;i < MOTION_BLUR_QUALITY;i++)
		{
			point += dir;
			color += texture(colortex3, point).rgb;
		}
		color /= MOTION_BLUR_QUALITY + 1;
	}
}	
#endif

void main()
{
    vec4 outColor = texture(colortex3, Texcoord);
	
	#ifdef MOTION_BLUR
    motionBlur(outColor.rgb);
	#endif
	#ifdef TAA
    	DoTAA(outColor.rgb);
	#endif
	
    texBuffer3 = outColor;
    
    texBuffer5 = vec4(outColor.rgb, 1.0);
}
/* DRAWBUFFERS:35 */